@@ -1,15 +1,18 @@
 """
 Map is a generic Map class from which all other Map classes inherit from.
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, division, print_function
+from sunpy.extern.six.moves import range
 
 #pylint: disable=E1101,E1121,W0404,W0613
-__authors__ = ["Russell Hewett, Stuart Mumford, Keith Hughitt, Steven Christe"]
+__authors__ = ["Russell Hewett, Stuart Mumford, Keith Hughitt, Steven Christe", "Jack Ireland"]
 __email__ = "stuart@mumford.me.uk"
 
 import warnings
 import inspect
+from abc import ABCMeta
 from copy import deepcopy
+from collections import OrderedDict
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -23,6 +26,7 @@ from .nddata_compat import NDDataCompat as NDData
 
 import sunpy.io as io
 import sunpy.wcs as wcs
+import sunpy.coordinates
 from sunpy.visualization import toggle_pylab, wcsaxes_compat
 from sunpy.sun import constants
 from sunpy.sun import sun
@@ -30,16 +34,20 @@ from sunpy.time import parse_time, is_time
 from sunpy.image.rescale import reshape_image_to_4d_superpixel
 from sunpy.image.rescale import resample as sunpy_image_resample
 
+from sunpy.extern import six
+
 import astropy.units as u
 
 from collections import namedtuple
 Pair = namedtuple('Pair', 'x y')
 
-__all__ = ['GenericMap']
 
 from sunpy import config
 TIME_FORMAT = config.get("general", "time_format")
 
+__all__ = ['GenericMap']
+
+
 """
 Questions
 ---------
@@ -48,7 +56,33 @@ or something else?)
 * Should 'center' be renamed to 'offset' and crpix1 & 2 be used for 'center'?
 """
 
+# GenericMap subclass registry.
+MAP_CLASSES = OrderedDict()
 
+
+class GenericMapMeta(ABCMeta):
+    """
+    Registration metaclass for `~sunpy.map.GenericMap`.
+
+    This class checks for the existance of a method named ``is_datasource_for``
+    when a subclass of `GenericMap` is defined. If it exists it will add that
+    class to the registry.
+    """
+
+    _registry = MAP_CLASSES
+
+    def __new__(mcls, name, bases, members):
+        cls = super(GenericMapMeta, mcls).__new__(mcls, name, bases, members)
+
+        # The registry contains the class as the key and the validation method
+        # as the item.
+        if 'is_datasource_for' in members:
+            mcls._registry[cls] = cls.is_datasource_for
+
+        return cls
+
+
+@six.add_metaclass(GenericMapMeta)
 class GenericMap(NDData):
     """
     A Generic spatially-aware 2D data array
@@ -148,7 +182,8 @@ class GenericMap(NDData):
 
         # Validate header
         # TODO: This should be a function of the header, not of the map
-        self._validate()
+        self._validate_meta()
+        self._shift = Pair(0 * u.arcsec, 0 * u.arcsec)
 
         if self.dtype == np.uint8:
             norm = None
@@ -203,20 +238,62 @@ scale:\t\t {scale}
         w2.wcs.crval = u.Quantity(self.reference_coordinate)
         w2.wcs.ctype = self.coordinate_system
         w2.wcs.pc = self.rotation_matrix
-        w2.wcs.cunit = self.units
+        w2.wcs.cunit = self.spatial_units
+        w2.wcs.dateobs = self.date.isoformat()
+        w2.heliographic_latitude = self.heliographic_latitude
+        w2.heliographic_longitude = self.heliographic_longitude
+        w2.dsun = self.dsun
 
         # Astropy WCS does not understand the SOHO default of "solar-x" and
         # "solar-y" ctypes.  This overrides the default assignment and
         # changes it to a ctype that is understood.  See Thompson, 2006, A.&A.,
         # 449, 791.
-        if w2.wcs.ctype[0].lower() == "solar-x":
+        if w2.wcs.ctype[0].lower() in ("solar-x", "solar_x"):
             w2.wcs.ctype[0] = 'HPLN-TAN'
 
-        if w2.wcs.ctype[1].lower() == "solar-y":
+        if w2.wcs.ctype[1].lower() in ("solar-y", "solar_y"):
             w2.wcs.ctype[1] = 'HPLT-TAN'
 
         return w2
 
+    @property
+    def coordinate_frame(self):
+        """
+        An `astropy.coordinates.BaseFrame` instance created from the coordinate
+        information for this Map.
+        """
+        return astropy.wcs.utils.wcs_to_celestial_frame(self.wcs)
+
+    def _as_mpl_axes(self):
+        """
+        Compatibility hook for Matplotlib and WCSAxes.
+        This functionality requires the WCSAxes package to work. The reason
+        we include this here is that it allows users to use WCSAxes without
+        having to explicitly import WCSAxes
+        With this method, one can do::
+
+            import matplotlib.pyplot as plt
+            import sunpy.map
+            amap = sunpy.map.Map('filename.fits')
+            fig = plt.figure()
+            ax = plt.subplot(projection=amap)
+            ...
+
+        and this will generate a plot with the correct WCS coordinates on the
+        axes. See http://wcsaxes.readthedocs.io for more information.
+        """
+        # This code is reused from Astropy
+
+        try:
+            from wcsaxes import WCSAxes
+        except ImportError:
+            raise ImportError("Using WCS instances as Matplotlib projections "
+                              "requires the WCSAxes package to be installed. "
+                              "See http://wcsaxes.readthedocs.io for more "
+                              "details.")
+        else:
+            return WCSAxes, {'wcs': self.wcs}
+
     # Some numpy extraction
     @property
     def dimensions(self):
@@ -352,6 +429,7 @@ scale:\t\t {scale}
     @property
     def xrange(self):
         """Return the X range of the image from edge to edge."""
+        #TODO: This should be reading from the WCS object
         xmin = self.center.x - self.dimensions[0] / 2. * self.scale.x
         xmax = self.center.x + self.dimensions[0] / 2. * self.scale.x
         return u.Quantity([xmin, xmax])
@@ -359,14 +437,14 @@ scale:\t\t {scale}
     @property
     def yrange(self):
         """Return the Y range of the image from edge to edge."""
+        #TODO: This should be reading from the WCS object
         ymin = self.center.y - self.dimensions[1] / 2. * self.scale.y
         ymax = self.center.y + self.dimensions[1] / 2. * self.scale.y
         return u.Quantity([ymin, ymax])
 
     @property
     def center(self):
-        """Returns the offset between the center of the Sun and the center of
-        the map."""
+        """The offset between the center of the Sun and the center of the map."""
         return Pair(wcs.get_center(self.dimensions[0], self.scale.x,
                                    self.reference_pixel.x,
                                    self.reference_coordinate.x),
@@ -374,6 +452,45 @@ scale:\t\t {scale}
                                    self.reference_pixel.y,
                                    self.reference_coordinate.y))
 
+    @property
+    def shifted_value(self):
+        """The total shift applied to the reference coordinate by past applications of
+        `~sunpy.map.GenericMap.shift`."""
+        return self._shift
+
+    @u.quantity_input(x=u.deg, y=u.deg)
+    def shift(self, x, y):
+        """Returns a map shifted by a specified amount to, for example, correct for a bad
+        map location. These values are applied directly to the `~sunpy.map.GenericMap.reference_coordinate`.
+        To check how much shift has already been applied see `~sunpy.map.GenericMap.shifted_value`
+
+        Parameters
+        ----------
+
+        x : `~astropy.units.Quantity`
+            The shift to apply to the X coordinate.
+
+        y : `~astropy.units.Quantity`
+            The shift to apply to the Y coordinate
+
+        Returns
+        -------
+        out : `~sunpy.map.GenericMap` or subclass
+            A new shifted Map.
+        """
+        new_map = deepcopy(self)
+        new_map._shift = Pair(self.shifted_value.x + x, self.shifted_value.y + y)
+
+        new_meta = self.meta.copy()
+
+        # Update crvals
+        new_meta['crval1'] = ((self.meta['crval1'] * self.spatial_units.x + x).to(self.spatial_units.x)).value
+        new_meta['crval2'] = ((self.meta['crval2'] * self.spatial_units.y + y).to(self.spatial_units.y)).value
+
+        new_map.meta = new_meta
+
+        return new_map
+
     @property
     def rsun_meters(self):
         """Radius of the sun in meters"""
@@ -432,26 +549,27 @@ scale:\t\t {scale}
 
     @property
     def reference_coordinate(self):
-        """Reference point WCS axes in data units (crval1/2)"""
-        return Pair(self.meta.get('crval1', 0.) * self.units.x,
-                    self.meta.get('crval2', 0.) * self.units.y)
+        """Reference point WCS axes in data units (i.e. crval1, crval2). This value
+        includes a shift if one is set."""
+        return Pair(self.meta.get('crval1', 0.) * self.spatial_units.x,
+                    self.meta.get('crval2', 0.) * self.spatial_units.y)
 
     @property
     def reference_pixel(self):
-        """Reference point axes in pixels (crpix1/2)"""
+        """Reference point axes in pixels (i.e. crpix1, crpix2)"""
         return Pair(self.meta.get('crpix1', (self.meta.get('naxis1') + 1) / 2.) * u.pixel,
                     self.meta.get('crpix2', (self.meta.get('naxis2') + 1) / 2.) * u.pixel)
 
     @property
     def scale(self):
-        """Image scale along the x and y axes in units/pixel (cdelt1/2)"""
+        """Image scale along the x and y axes in units/pixel (i.e. cdelt1, cdelt2)"""
         #TODO: Fix this if only CDi_j matrix is provided
-        return Pair(self.meta.get('cdelt1', 1.) * self.units.x / u.pixel,
-                    self.meta.get('cdelt2', 1.) * self.units.y / u.pixel)
+        return Pair(self.meta.get('cdelt1', 1.) * self.spatial_units.x / u.pixel,
+                    self.meta.get('cdelt2', 1.) * self.spatial_units.y / u.pixel)
 
     @property
-    def units(self):
-        """Image coordinate units along the x and y axes (cunit1/2)."""
+    def spatial_units(self):
+        """Image coordinate units along the x and y axes (i.e. cunit1, cunit2)."""
         return Pair(u.Unit(self.meta.get('cunit1', 'arcsec')),
                     u.Unit(self.meta.get('cunit2', 'arcsec')))
 
@@ -523,16 +641,29 @@ scale:\t\t {scale}
         cmap_string = self.observatory + self.meta['detector'] + str(int(self.wavelength.to('angstrom').value))
         return cmap_string.lower()
 
-    def _validate(self):
-        """Validates the meta-information associated with a Map.
+    def _validate_meta(self):
+        """
+        Validates the meta-information associated with a Map.
 
-        This function includes very basic validation checks which apply to
+        This method includes very basic validation checks which apply to
         all of the kinds of files that SunPy can read. Datasource-specific
         validation should be handled in the relevant file in the
-        sunpy.map.sources package."""
-#        if (self.dsun <= 0 or self.dsun >= 40 * constants.au):
-#            raise InvalidHeaderInformation("Invalid value for DSUN")
-        pass
+        sunpy.map.sources package.
+
+        Allows for default unit assignment for:
+            CUNIT1, CUNIT2, WAVEUNIT
+
+        """
+
+        warnings.simplefilter('always', Warning)
+
+        for meta_property in ('cunit1', 'cunit2', 'waveunit'):
+            if (self.meta.get(meta_property) and
+                u.Unit(self.meta.get(meta_property),
+                       parse_strict='silent').physical_type == 'unknown'):
+
+                warnings.warn("Unknown value for "+meta_property.upper(), Warning)
+
 
 # #### Data conversion routines #### #
 
@@ -601,14 +732,18 @@ scale:\t\t {scale}
         """
         x, y = self.wcs.wcs_pix2world(x, y, origin)
 
-        # WCS always outputs degrees.
-        x = u.Quantity(x, u.deg)
-        y = u.Quantity(y, u.deg)
+        # If the wcs is celestial it is output in degress
+        if self.wcs.is_celestial:
+            x = u.Quantity(x, u.deg)
+            y = u.Quantity(y, u.deg)
+        else:
+            x = u.Quantity(x, self.spatial_units.x)
+            y = u.Quantity(y, self.spatial_units.y)
 
         x = Longitude(x, wrap_angle=180*u.deg)
         y = Latitude(y)
 
-        return x.to(self.units.x), y.to(self.units.y)
+        return x.to(self.spatial_units.x), y.to(self.spatial_units.y)
 
 
 # #### I/O routines #### #
@@ -814,8 +949,8 @@ scale:\t\t {scale}
         # Calculate the needed padding or unpadding
         diff = np.asarray(np.ceil((extent - new_map.data.shape) / 2)).ravel()
         # Pad the image array
-        pad_x = np.int(np.max((diff[1], 0)))
-        pad_y = np.int(np.max((diff[0], 0)))
+        pad_x = int(np.max((diff[1], 0)))
+        pad_y = int(np.max((diff[0], 0)))
         new_map.data = np.pad(new_map.data,
                               ((pad_y, pad_y), (pad_x, pad_x)),
                               mode='constant',
@@ -961,14 +1096,14 @@ scale:\t\t {scale}
         if ((isinstance(range_a, u.Quantity) and isinstance(range_b, u.Quantity)) or
             (hasattr(range_a, 'unit') and hasattr(range_b, 'unit'))):
 
-            if (range_a.unit.is_equivalent(self.units.x) and
-                range_b.unit.is_equivalent(self.units.y)):
+            if (range_a.unit.is_equivalent(self.spatial_units.x) and
+                range_b.unit.is_equivalent(self.spatial_units.y)):
                 units = 'data'
             elif range_a.unit.is_equivalent(u.pixel) and range_b.unit.is_equivalent(u.pixel):
                 units = 'pixels'
             else:
                 raise u.UnitsError("range_a and range_b but be "
-                                   "in units convertable to {} or {}".format(self.units['x'],
+                                   "in units convertable to {} or {}".format(self.spatial_units['x'],
                                                                              u.pixel))
         else:
             raise TypeError("Arguments range_a and range_b to function submap "
@@ -1006,13 +1141,9 @@ scale:\t\t {scale}
             x_pixels = range_a.value
             y_pixels = range_b.value
         else:
-            raise ValueError(
-                "Invalid unit. Must be one of 'data' or 'pixels'")
+            raise ValueError("Invalid unit. Must be one of 'data' or 'pixels'")
 
-        x_pixels.sort()
-        y_pixels.sort()
-
-        # Sort the pixel values so we are always slicing in the correct direction
+        # Sort the pixel values so we always slice in the correct direction
         x_pixels.sort()
         y_pixels.sort()
 
@@ -1028,8 +1159,8 @@ scale:\t\t {scale}
         y_pixels[np.greater(y_pixels, self.data.shape[0])] = self.data.shape[0]
 
         # Get ndarray representation of submap
-        xslice = slice(x_pixels[0], x_pixels[1])
-        yslice = slice(y_pixels[0], y_pixels[1])
+        xslice = slice(int(x_pixels[0]), int(x_pixels[1]))
+        yslice = slice(int(y_pixels[0]), int(y_pixels[1]))
         new_data = self.data[yslice, xslice].copy()
 
         # Make a copy of the header with updated centering information
@@ -1041,24 +1172,33 @@ scale:\t\t {scale}
 
         # Create new map instance
         new_map.data = new_data
+        if self.mask is not None:
+            new_map.mask = self.mask[yslice, xslice].copy()
+
         return new_map
 
-    @u.quantity_input(dimensions=u.pixel)
-    def superpixel(self, dimensions, method='sum'):
-        """Returns a new map consisting of superpixels formed from the
-        original data.  Useful for increasing signal to noise ratio in images.
+    @u.quantity_input(dimensions=u.pixel, offset=u.pixel)
+    def superpixel(self, dimensions, offset=(0, 0)*u.pixel, func=np.sum):
+        """Returns a new map consisting of superpixels formed by applying
+        'func' to the original map data.
 
         Parameters
         ----------
         dimensions : tuple
             One superpixel in the new map is equal to (dimension[0],
-            dimension[1]) pixels of the original map
+            dimension[1]) pixels of the original map.
             Note: the first argument corresponds to the 'x' axis and the second
             argument corresponds to the 'y' axis.
-        method : {'sum' | 'average'}
-            What each superpixel represents compared to the original data
-                * sum - add up the original data
-                * average - average the sum over the number of original pixels
+        offset : tuple
+            Offset from (0,0) in original map pixels used to calculate where
+            the data used to make the resulting superpixel map starts.
+        func : function applied to the original data
+            The function 'func' must take a numpy array as its first argument,
+            and support the axis keyword with the meaning of a numpy axis
+            keyword (see the description of `~numpy.sum` for an example.)
+            The default value of 'func' is `~numpy.sum`; using this causes
+            superpixel to sum over (dimension[0], dimension[1]) pixels of the
+            original map.
 
         Returns
         -------
@@ -1072,25 +1212,31 @@ scale:\t\t {scale}
 
         # Note: because the underlying ndarray is transposed in sense when
         #   compared to the Map, the ndarray is transposed, resampled, then
-        #   transposed back
+        #   transposed back.
         # Note: "center" defaults to True in this function because data
-        #   coordinates in a Map are at pixel centers
+        #   coordinates in a Map are at pixel centers.
 
-        # Make a copy of the original data and perform reshaping
-        reshaped = reshape_image_to_4d_superpixel(self.data.copy(),
-                                                  [dimensions.value[1], dimensions.value[0]])
-        if method == 'sum':
-            new_data = reshaped.sum(axis=3).sum(axis=1)
-        elif method == 'average':
-            new_data = ((reshaped.sum(axis=3).sum(axis=1)) /
-                    np.float32(dimensions[0] * dimensions[1]))
+        if (offset.value[0] < 0) or (offset.value[1] < 0):
+            raise ValueError("Offset is strictly non-negative.")
+
+        # Make a copy of the original data, perform reshaping, and apply the
+        # function.
+        if self.mask is not None:
+            reshaped = reshape_image_to_4d_superpixel(np.ma.array(self.data.copy(), mask=self.mask),
+                                                      [dimensions.value[1], dimensions.value[0]],
+                                                      [offset.value[1], offset.value[0]])
+        else:
+            reshaped = reshape_image_to_4d_superpixel(self.data.copy(),
+                                                      [dimensions.value[1], dimensions.value[0]],
+                                                      [offset.value[1], offset.value[0]])
+        new_array = func(func(reshaped, axis=3), axis=1)
 
         # Update image scale and number of pixels
         new_map = deepcopy(self)
         new_meta = new_map.meta
 
-        new_nx = (self.dimensions[0] / dimensions[0]).value
-        new_ny = (self.dimensions[1] / dimensions[1]).value
+        new_nx = new_array.shape[1]
+        new_ny = new_array.shape[0]
 
         # Update metadata
         new_meta['cdelt1'] = (dimensions[0] * self.scale.x).value
@@ -1102,11 +1248,16 @@ scale:\t\t {scale}
             new_meta['CD2_2'] *= dimensions[1].value
         new_meta['crpix1'] = (new_nx + 1) / 2.
         new_meta['crpix2'] = (new_ny + 1) / 2.
-        new_meta['crval1'] = self.center.x.value
-        new_meta['crval2'] = self.center.y.value
+        new_meta['crval1'] = self.center.x.to(self.spatial_units.x).value + 0.5*(offset[0]*self.scale.x).to(self.spatial_units.x).value
+        new_meta['crval2'] = self.center.y.to(self.spatial_units.y).value + 0.5*(offset[1]*self.scale.y).to(self.spatial_units.y).value
 
         # Create new map instance
-        new_map.data = new_data
+        if self.mask is not None:
+            new_map.data = np.ma.getdata(new_array)
+            new_map.mask = np.ma.getmask(new_array)
+        else:
+            new_map.data = new_array
+            new_map.mask = None
         return new_map
 
 # #### Visualization #### #
@@ -1150,13 +1301,13 @@ scale:\t\t {scale}
 
         b0 = self.heliographic_latitude.to(u.deg).value
         l0 = self.heliographic_longitude.to(u.deg).value
-        units = self.units
+        units = self.spatial_units
 
-        #Prep the plot kwargs
-        plot_kw = {'color':'white',
-                   'linestyle':'dotted',
-                   'zorder':100,
-                   'transform':transform}
+        # Prep the plot kwargs
+        plot_kw = {'color': 'white',
+                   'linestyle': 'dotted',
+                   'zorder': 100,
+                   'transform': transform}
         plot_kw.update(kwargs)
 
         hg_longitude_deg = np.linspace(-180, 180, num=361) + l0
@@ -1237,9 +1388,105 @@ scale:\t\t {scale}
 
         return [circ]
 
+    @u.quantity_input(bottom_left=u.deg, width=u.deg, height=u.deg)
+    def draw_rectangle(self, bottom_left, width, height, axes=None, **kwargs):
+        """
+        Draw a rectangle defined in world coordinates on the plot.
+
+        Parameters
+        ----------
+
+        bottom_left : `astropy.units.Quantity`
+            The bottom left corner of the rectangle.
+
+        width : `astropy.units.Quantity`
+            The width of the rectangle.
+
+        height : `astropy.units.Quantity`
+            The height of the rectangle.
+
+        axes : `matplotlib.axes.Axes`
+            The axes on which to plot the rectangle, defaults to the current axes.
+
+        Returns
+        -------
+
+        rect : `list`
+            A list containing the `~matplotlib.patches.Rectangle` object, after it has been added to ``axes``.
+
+        Notes
+        -----
+
+        Extra keyword arguments to this function are passed through to the
+        `~matplotlib.patches.Rectangle` instance.
+
+        """
+
+        if not axes:
+            axes = plt.gca()
+
+        if wcsaxes_compat.is_wcsaxes(axes):
+            axes_unit = u.deg
+        else:
+            axes_unit = self.spatial_units[0]
+
+        bottom_left = bottom_left.to(axes_unit).value
+        width = width.to(axes_unit).value
+        height = height.to(axes_unit).value
+
+        kwergs = {'transform':wcsaxes_compat.get_world_transform(axes),
+                  'color':'white',
+                  'fill':False}
+        kwergs.update(kwargs)
+        rect = plt.Rectangle(bottom_left, width, height, **kwergs)
+
+        axes.add_artist(rect)
+
+        return [rect]
+
+    @u.quantity_input(levels=u.percent)
+    def draw_contours(self, levels, axes=None, **contour_args):
+        """
+        Draw contours of the data
+
+        Parameters
+        ----------
+
+        levels : `~astropy.units.Quantity`
+            A list of numbers indicating the level curves to draw given in
+            percent.
+
+        axes : `matplotlib.axes.Axes`
+            The axes on which to plot the rectangle, defaults to the current
+            axes.
+
+        Returns
+        -------
+
+        cs : `list`
+            The `~matplotlib.QuadContourSet` object, after it has been added to
+            ``axes``.
+
+        Notes
+        -----
+
+        Extra keyword arguments to this function are passed through to the
+        `~matplotlib.pyplot.contour` function.
+
+        """
+        if not axes:
+            axes = wcsaxes_compat.gca_wcs(self.wcs)
+
+        # TODO: allow for use of direct input of contours but requires units of
+        # map flux which is not yet implemented
+
+        cs = axes.contour(self.data, 0.01 * levels.to('percent').value * self.data.max(),
+                          **contour_args)
+        return cs
+
     @toggle_pylab
     def peek(self, draw_limb=False, draw_grid=False,
-                   colorbar=True, basic_plot=False, **matplot_args):
+             colorbar=True, basic_plot=False, **matplot_args):
         """Displays the map in a new figure
 
         Parameters
@@ -1288,10 +1535,10 @@ scale:\t\t {scale}
         if isinstance(draw_grid, bool):
             if draw_grid:
                 self.draw_grid(axes=axes)
-        elif isinstance(draw_grid, (int, long, float)):
+        elif isinstance(draw_grid, u.Quantity):
             self.draw_grid(axes=axes, grid_spacing=draw_grid)
         else:
-            raise TypeError("draw_grid should be bool, int, long or float")
+            raise TypeError("draw_grid should be a bool or an astropy Quantity.")
 
         figure.show()
 
@@ -1337,9 +1584,14 @@ scale:\t\t {scale}
             warnings.warn("This map is not properly oriented. Plot axes may be incorrect",
                           Warning)
 
+        if not wcsaxes_compat.is_wcsaxes(axes) and wcsaxes_compat.HAVE_WCSAXES:
+            warnings.warn("WCSAxes is installed but not being used."
+                          " Plots may not have the expected behaviour.",
+                          Warning)
+
         # Normal plot
         imshow_args = deepcopy(self.plot_settings)
-        if imshow_args.has_key('title'):
+        if 'title' in imshow_args:
             plot_settings_title = imshow_args.pop('title')
         else:
             plot_settings_title = self.name
@@ -1353,15 +1605,15 @@ scale:\t\t {scale}
 
             # x-axis label
             if self.coordinate_system.x == 'HG':
-                xlabel = 'Longitude [{lon}]'.format(lon=self.units.x)
+                xlabel = 'Longitude [{lon}]'.format(lon=self.spatial_units.x)
             else:
-                xlabel = 'X-position [{xpos}]'.format(xpos=self.units.x)
+                xlabel = 'X-position [{xpos}]'.format(xpos=self.spatial_units.x)
 
             # y-axis label
             if self.coordinate_system.y == 'HG':
-                ylabel = 'Latitude [{lat}]'.format(lat=self.units.y)
+                ylabel = 'Latitude [{lat}]'.format(lat=self.spatial_units.y)
             else:
-                ylabel = 'Y-position [{ypos}]'.format(ypos=self.units.y)
+                ylabel = 'Y-position [{ypos}]'.format(ypos=self.spatial_units.y)
 
             axes.set_xlabel(xlabel)
             axes.set_ylabel(ylabel)
